/*globals require*/
require([
    'jquery',
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/presentation/ui/multiselect-test.html',

    'de_epages/presentation/ui/multiselect',
    '$ready!'
], function ($, QUnit, fixture) {
    'use strict';

    QUnit.module("de_epages/presentation/ui/multiselect", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest('Checking searchbar\'s methods', 7, function () {
        $.each([
            '_create',
            '_handleButtonClick',
            '_handleCheckboxClick',
            '_handleBodyClick',
            '_disableOption',
            '_activateOption',
            'destroy'
        ], function () {
            QUnit.ok(
                $.isFunction($.ui.presentationUiMultiselect.prototype[this]),
                this + ' is a method of the widget'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest('Check HTML structure and visible options.', function () {

        var multiselectElem = $('#test'),
            options = {
              tableHeader: 'tableHead',
              id: 'TESTID',
              values:[{
                name: 'test1',
                value: 'test1'
              },{
                name: 'test2',
                value: 'test2',
                checked: true
              },{
                name: 'test3',
                value: 'test3',
                checked: true,
                selection: 1
              },{
                name: 'test4',
                value: 'test4',
                checked: true,
                selection: 2
              },{
                name: 'test5',
                value: 'test5',
                selection: 2
              }]
            },
            rows,
            firstRow;

        multiselectElem.presentationUiMultiselect(options);

        // wait for initialization
        window.setTimeout(function() {
            QUnit.ok(true, '================== [Test Initialization of Widget] ==================');
            // check presence of searchbar widget
            QUnit.equal(
                multiselectElem.data('uiPresentationUiMultiselect') instanceof $.Widget,
                true,
                "Multiselect widget is created and instance of jQuery.Widget."
            );

            rows = multiselectElem.find('.ep-multiselect-list-item');

            QUnit.equal(
                rows.length,
                options.values.length,
                'All ' + options.values.length + ' values were rendered.'
            );

            rows.each(function(index, elem) {
                var $elem = $(elem);

                QUnit.equal(
                    $elem.find('input').length,
                    3,
                    'Row ' + index + 'contains 3 inputs.'
                );

                QUnit.equal(
                    $elem.find('input').get(0).checked,
                    !!options.values[index].checked,
                    'Checkbox is ' + (options.values[index].checked ? '' : 'not ') + 'checked.'
                );

                QUnit.equal(
                    $elem.find('input').get(1).checked,
                    options.values[index].selection === 1 && !!options.values[index].checked,
                    'Radiobutton 1 is ' + ((options.values[index].selection === 1 && options.values[index].checked) ? '' : 'not ') + 'checked.'
                );

                QUnit.equal(
                    $elem.find('input').get(2).checked,
                    options.values[index].selection === 2 && !!options.values[index].checked,
                    'Radiobutton 2 is ' + ((options.values[index].selection === 2 && options.values[index].checked) ? '' : 'not ') + 'checked.'
                );

                QUnit.equal(
                    $elem.find('input').get(1).disabled,
                    !options.values[index].checked,
                    'Radiobutton 1 is ' + (!options.values[index].checked ? '' : 'not ') + 'disabled.'
                );

                QUnit.equal(
                    $elem.find('input').get(2).disabled,
                    !options.values[index].checked,
                    'Radiobutton 2 is ' + (!options.values[index].checked ? '' : 'not ') + 'disabled.'
                );
            });

            QUnit.ok(true, '================== [Test Functionality of Widget] ==================');

            firstRow = rows.eq(0);

            firstRow.find('input[type=checkbox]').trigger('click');

            window.setTimeout(function() {
                QUnit.equal(
                    firstRow.find('input').get(1).disabled,
                    false,
                    'Radiobutton 1 is not disabled.'
                );

                QUnit.equal(
                    firstRow.find('input').get(2).disabled,
                    false,
                    'Radiobutton 2 is not disabled.'
                );

                firstRow.find('input[type=radio]').eq(1).trigger('click');

                firstRow.find('input[type=checkbox]').trigger('click');

                window.setTimeout(function() {
                    QUnit.equal(
                        firstRow.find('input').get(1).disabled,
                        true,
                        'Radiobutton 1 is disabled.'
                    );

                    QUnit.equal(
                        firstRow.find('input').get(2).disabled,
                        true,
                        'Radiobutton 2 is disabled.'
                    );

                    QUnit.equal(
                        firstRow.find('input').get(1).checked,
                        false,
                        'Radiobutton 1 is not checked.'
                    );

                    QUnit.equal(
                        firstRow.find('input').get(2).checked,
                        false,
                        'Radiobutton 2 is not checked.'
                    );

                    QUnit.ok(true, '================== [Test Destroy Method] ==================');

                    // destroy searchbar widget
                    multiselectElem.presentationUiMultiselect('destroy');

                    // check presence of multiselect widget
                    QUnit.equal(
                        multiselectElem.data('uiPresentationUiMultiselect') instanceof $.Widget,
                        false,
                        "Multiselect widget was destroyed."
                    );

                    QUnit.start();
                }, 100);
            }, 100);
        }, 100);

    });
});