/*
 * @copyright        © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module            ep.fn.holdOnTop
 */

define("ep/fn/holdontop", [
    "jquery",
    "ep"
], function($, ep) {

    var win = $(window),
        body = $('body'),
        elementStack = [],
        eventTimeout = [],

        holdOnTopUnity = function(i) {
            if (eventTimeout[i]) {
                return;
            }

            eventTimeout[i] = true;

            var self = this,
                o = self.options,
                elem = self.element,
                bodyPos = {
                    top: window.pageYOffset,
                    left: window.pageXOffset
                },
                setCSS = {
                    position: 'fixed'
                },
                elemPos,
                bodyHeight,
                bodyWidth,
                titleBarHeight = 0;

            if (!self.placeholder) {
                self.placeholder = $('<div>').insertBefore(elem);
            }

            self.placeholder.hide();

            elem.attr('style', self.restore)
                .removeClass('holdOnTop');

            // execute callback after holdOnTop was disabled
            if (self.callbacks && self.callbacks.onAfterRemoveHoldOnTop && typeof self.callbacks.onAfterRemoveHoldOnTop === 'function') {
                self.callbacks.onAfterRemoveHoldOnTop({
                    elem: this.element
                });
            }

            elemPos = elem.offset();

            if (o == 'bottom') {
                bodyHeight = body.outerHeight(true);

                elemPos.bottom = bodyHeight - elem.outerHeight(true) - elemPos.top + parseFloat(elem.css("padding-bottom")) + parseFloat(elem.css("margin-bottom"));
                bodyPos.bottom = bodyHeight - win.height() - bodyPos.top;
                setCSS.left = '0';

            } else if (o == 'right') {
                bodyWidth = body.outerWidth(true);

                elemPos.right = bodyWidth - elem.outerWidth(true) - elemPos.left;
                bodyPos.right = bodyWidth - win.width() - bodyPos.left;

            } else if (o == 'top') {
                titleBarHeight = 50; //height of mbo title bar in pixel
            }

            if ((bodyPos[o] + titleBarHeight) > elemPos[o]) {
                setCSS[o] = 0 + 'px';
                self.placeholder
                    .width(elem.outerWidth(true))
                    .height(elem.outerHeight(true))
                    .show();

                elem.css(setCSS)
                    .addClass('holdOnTop');

                // execute callback after holdOnTop was enabled
                if (self.callbacks && self.callbacks.onAfterEnableHoldOnTop && typeof self.callbacks.onAfterEnableHoldOnTop === 'function') {
                    self.callbacks.onAfterEnableHoldOnTop({
                        elem: this.element
                    });
                }
            }

            setTimeout(function() {
                eventTimeout[i] = false;
            }, 10);
        },

        holdOnTop = function(i) {
            var self = this,
                o = self.options,
                elem = self.element,
                elemPos = elem
                    .attr('style', self.restore)
                    .offset(),

                bodyPos = {
                    top: window.pageYOffset,
                    left: window.pageXOffset
                },

                setCSS = {
                    position: 'fixed'
                };

            if (!self.placeholder) {
                self.placeholder = $('<div>').insertBefore(elem);
            }
            self.placeholder.hide();

            if (o == 'bottom') {
                var bodyHeight = body.outerHeight(true);

                elemPos.bottom = bodyHeight - elem.outerHeight(true) - elemPos.top;
                bodyPos.bottom = bodyHeight - win.height() - bodyPos.top;
                setCSS.left = '0';

            } else if (o == 'right') {
                var bodyWidth = body.outerWidth(true);

                elemPos.right = bodyWidth - elem.outerWidth(true) - elemPos.left;
                bodyPos.right = bodyWidth - win.width() - bodyPos.left;
            }

            if (bodyPos[o] > elemPos[o]) {
                setCSS[o] = 0 + 'px';
                self.placeholder
                    .width(elem.outerWidth(true))
                    .height(elem.outerHeight(true))
                    .show();

                elem.css(setCSS)
                    .addClass('holdOnTop');

            } else if (bodyPos[o] < elemPos[o]) {
                elem.removeClass('holdOnTop');
            }
        },

        onBeforeEvent = function() {
            var fn = function (i, elem) {
                var self = this
                    elem = self.element;

                if (!self.placeholder) {
                    self.placeholder = $('<div>').insertBefore(elem);
                }

                self.placeholder.hide();

                elem.attr('style', self.restore)
                    .removeClass('holdOnTop');
            };

            $.each(elementStack, fn);
        },

        onEvent = function () {
            var fn = ep.config.unity
                ? holdOnTopUnity
                : holdOnTop;

            $.each(elementStack, fn);
        };

    ep.fn.holdOnTop = function(options, callbacks) {
        this.each(function() {
            var elem = $(this);

            elementStack.push({
                element: elem,
                options: options || 'top',
                callbacks: callbacks || {},
                restore: elem.attr('style') || ""
            });
        });

        onEvent();

        return this;
    };

    win.on('scroll resize accordionactivate accordionchange', onEvent);
    win.on('accordionbeforeactivate', onBeforeEvent);

    return ep;
});
