/*global define,window,tree*/

define([
    'testsuite/qunit',
    'jquery',
    'de_epages/mediagallery/filemanager/tree-view',
    'de_epages/mediagallery/filemanager/folder-model',

    '$ready!'
], function (QUnit, $, TreeView, FolderModel) {
    'use strict';

    QUnit.module('de_epages/mediagallery/filemanager/tree-view', {
        setup: function () {
            var filemanager = $('<div class="filemanager" />').appendTo('#qunit-fixture'),
                data = {
                    "files": [],
                    "children": [{
                        "files": [],
                        "children": [],
                        "metadata": {
                            "displayname": "test",
                            "type": "MediaGallery",
                            "path": "/neu/test"
                        },
                        "data": "test",
                        "state": null
                    }],
                    "metadata": {
                        "displayname": "neu",
                        "type": "MediaGallery",
                        "path": "/neu"
                    },
                    "data": "neu",
                    "state": null
                },
                model = new FolderModel(data);

            window.tree = new TreeView({model: model, treeData: data, currentFolder: 'neu'});

            $('.filemanager').append(tree.render().el);

            QUnit.strictEqual(
                $('.ep-mediagallery-treeView').length,
                1,
                'Create tree view and append it to filemanager'
            );

            tree
                .on('folderEditable', function (editable) {
                    QUnit.strictEqual(
                        editable,
                        false,
                        'Trigger event folderEditable on tree view'
                    );
                })
                .on('folderChanged', function (model) {
                    QUnit.strictEqual(
                        model.get('data'),
                        'neu',
                        'Trigger event folderChanged on tree view'
                    );
                });
        },
        teardown: function () {
            $('.ep-mediagallery-treeView').remove();

            QUnit.strictEqual(
                $('.ep-mediagallery-treeView').length,
                0,
                'Remove tree view from filemanager container'
            );
        }
    });

    QUnit.test('view methods', function () {
        $.each(['initialize', 'render', 'addTreeNode', 'renameTreeNode', 'removeTreeNode', 'setActiveNode', 'onHeaderClick', 'onLinkClick', 'setState'], function (i, method) {
            QUnit.strictEqual(
                $.isFunction(tree[method]),
                true,
                method + ' is a method of tree view'
            );
        });
    });

    QUnit.test('view functionality', function () {
        var model;

        QUnit.strictEqual(
            tree.$('.jstree-leaf').length,
            1,
            'Folder neu contains one folder'
        );

        tree.addTreeNode({
            data: {
                files: [],
                children: [],
                metadata: {
                    type: "MediaGallery",
                    path: "/Neuer_Ordner"
                },
                data: "Neuer_Ordner",
                state: null,
                id: "MediaGallery/Neuer_Ordner"
            },
            id: "MediaGallery",
            path: ""
        });

        QUnit.strictEqual(
            tree.$('.jstree-leaf').length,
            2,
            'Folder neu contains now two folders'
        );

        QUnit.strictEqual(
            tree.$('li:last').attr('id'),
            'MediaGallery/Neuer_Ordner',
            'Create folder Neuer_Ordner'
        );

        model = new FolderModel({
            files: [],
            children: [],
            metadata: {
                path: "/bilder",
                type: "MediaGallery"
            },
            data: "bilder",
            state: null,
            attr: {
                id: "MediaGallery/Neuer_Ordner",
                title: "Neuer_Ordner"
            },
            modelType: "folder",
            id: "MediaGallery/bilder"
        });

        tree.renameTreeNode({
            model: model,
            prevID: "MediaGallery/Neuer_Ordner"
        });

        QUnit.strictEqual(
            tree.$('li:last').attr('id'),
            'MediaGallery/bilder',
            'Rename folder Neuer_Ordner to bilder'
        );

        tree.setActiveNode('MediaGallery/bilder');

        QUnit.strictEqual(
            tree.$('a.active').length,
            1,
            'Mark folder bilder as active'
        );

        tree.$('.ep-mediagallery-treeHead').trigger('click');
        tree.$('.ep-mediagallery-tree a:first').trigger('click');
        tree.setState();
        tree.removeTreeNode({id: 'MediaGallery/bilder'});

        QUnit.strictEqual(
            tree.$('.jstree-leaf').length,
            1,
            'Remove folder bilder'
        );

    });

});